@@ -8,6 +8,7 @@
 #define ML_ERROR(cause) do { } while(0)
 
 static int qgammaf (gnm_float x, GnmQuad *mant, int *exp2);
+static void pochhammer_small_n (gnm_float x, gnm_float n, GnmQuad *res);
 
 
 /* Compute  gnm_log(gamma(a+1))  accurately also for small a (0 < a < 0.5). */
@@ -515,6 +516,24 @@ qbetaf (gnm_float a, gnm_float b, GnmQuad *mant, int *exp2)
 		return 0;
 	}
 
+	if (b > a) {
+		gnm_float s = a;
+		a = b;
+		b = s;
+	}
+
+	if (a > 1 && gnm_abs (b) < 1) {
+		void *state;
+		if (qgammaf (b, &mb, &eb))
+			return 1;
+		state = gnm_quad_start ();
+		pochhammer_small_n (a, b, &ma);
+		gnm_quad_div (mant, &mb, &ma);
+		gnm_quad_end (state);
+		*exp2 = eb;
+		return 0;
+	}
+
 	if (!qgammaf (a, &ma, &ea) &&
 	    !qgammaf (b, &mb, &eb) &&
 	    !qgammaf (ab, &mab, &eab)) {
@@ -602,7 +621,7 @@ gnm_lbeta3 (gnm_float a, gnm_float b, int *sign)
  *
  *   Gamma(x) = sqrt(2Pi) * x^(x-1/2) * exp(-x) * E(x)
  *
- * x should be >20 and the result is, roughly, 1+1/(12x).
+ * x should be >0 and the result is, roughly, 1+1/(12x).
  */
 static void
 gamma_error_factor (GnmQuad *res, const GnmQuad *x)
@@ -629,21 +648,65 @@ gamma_error_factor (GnmQuad *res, const GnmQuad *x)
 		GNM_const(86684309913600.),
 		GNM_const(514904800886784000.)
 	};
-	GnmQuad zn;
+	GnmQuad zn, xpn;
 	int i;
+	gnm_float xval = gnm_quad_value (x);
+	int n;
+
+	g_return_if_fail (xval > 0);
+
+	// We want x >= 20 for the asymptotic expansion
+	n = (xval < 20) ? (int)gnm_floor (21 - xval) : 0;
+	gnm_quad_init (&xpn, n);
+	gnm_quad_add (&xpn, &xpn, x);
 
 	gnm_quad_init (&zn, 1);
 	*res = zn;
 
 	for (i = 0; i < (int)G_N_ELEMENTS (num); i++) {
 		GnmQuad t, c;
-		gnm_quad_mul (&zn, &zn, x);
+		gnm_quad_mul (&zn, &zn, &xpn);
 		gnm_quad_init (&c, den[i]);
 		gnm_quad_mul (&t, &zn, &c);
 		gnm_quad_init (&c, num[i]);
 		gnm_quad_div (&t, &c, &t);
 		gnm_quad_add (res, res, &t);
 	}
+
+	if (n > 0) {
+		int i;
+		GnmQuad en, xxn, xph;
+
+		// Gamma(x) = sqrt(2Pi) * x^(x-1/2) * exp(-x) * E(x)
+		// Gamma(x+n) = sqrt(2Pi) * (x+n)^(x+n-1/2) * exp(-x-n) * E(x+n)
+
+		// E(x+n) / E(x) =
+		// Gamma(x+n)/Gamma(x) * (x^(x-1/2) * exp(-x)) / ((x+n)^(x+n-1/2) * exp(-x-n)) =
+		// (x*(x+1)*...*(x+n-1)) * exp(n) * (x/(x+n))^(x-1/2) / (x+n)^n =
+		// ((x+1)*...*(x+n-1)) * exp(n) * (x/(x+n))^(x+1/2) / (x+n)^(n-1) =
+		// ((x+1)/(x+n)*...*(x+n-1)/(x+n)) * exp(n) * (x/(x+n))^(x+1/2)
+
+		for (i = 1; i < n; i++) {
+			// *= (x+i)/(x+n)
+			GnmQuad xpi;
+			gnm_quad_init (&xpi, i);
+			gnm_quad_add (&xpi, &xpi, x);
+			gnm_quad_div (res, res, &xpi);
+			gnm_quad_mul (res, res, &xpn);
+		}
+
+		// /= exp(n)
+		gnm_quad_init (&en, n);
+		gnm_quad_exp (&en, NULL, &en);
+		gnm_quad_div (res, res, &en);
+
+		// /= (x/(x+n))^(x+1/2)
+		gnm_quad_init (&xph, 0.5);
+		gnm_quad_add (&xph, &xph, x);
+		gnm_quad_div (&xxn, x, &xpn);
+		gnm_quad_pow (&xxn, NULL, &xxn, &xph);
+		gnm_quad_div (res, res, &xxn);
+	}
 }
 
 /* ------------------------------------------------------------------------- */
@@ -655,7 +718,7 @@ pochhammer_small_n (gnm_float x, gnm_float n, GnmQuad *res)
 	gnm_float r;
 	gboolean debug = FALSE;
 
-	g_return_if_fail (x >= 20);
+	g_return_if_fail (x >= 1);
 	g_return_if_fail (gnm_abs (n) <= 1);
 
 	/*
@@ -1187,116 +1250,128 @@ static const guint32 lanczos_denom[G_N_ELEMENTS(lanczos_num)] = {
 };
 
 void
-complex_gamma (complex_t *dst, complex_t const *src)
+complex_gamma (gnm_complex *dst, gnm_complex const *src)
 {
-	if (complex_real_p (src)) {
-		complex_init (dst, gnm_gamma (src->re), 0);
+	if (gnm_complex_real_p (src)) {
+		gnm_complex_init (dst, gnm_gamma (src->re), 0);
 	} else if (src->re < 0) {
 		/* Gamma(z) = pi / (sin(pi*z) * Gamma(-z+1)) */
-		complex_t a, b, mz;
+		gnm_complex a, b, mz;
 
-		complex_init (&mz, -src->re, -src->im);
+		gnm_complex_init (&mz, -src->re, -src->im);
 		complex_fact (&a, &mz);
 
-		complex_init (&b,
+		gnm_complex_init (&b,
 			      M_PIgnum * gnm_fmod (src->re, 2),
 			      M_PIgnum * src->im);
 		/* Hmm... sin overflows when b.im is large.  */
-		complex_sin (&b, &b);
+		gnm_complex_sin (&b, &b);
 
-		complex_mul (&a, &a, &b);
+		gnm_complex_mul (&a, &a, &b);
 
-		complex_init (&b, M_PIgnum, 0);
+		gnm_complex_init (&b, M_PIgnum, 0);
 
-		complex_div (dst, &b, &a);
+		gnm_complex_div (dst, &b, &a);
 	} else {
-		complex_t zmh, zmhd2, zmhpg, f, f2, p, q, pq;
+		gnm_complex zmh, zmhd2, zmhpg, f, f2, p, q, pq;
 		int i;
 
 		i = G_N_ELEMENTS(lanczos_num) - 1;
-		complex_init (&p, lanczos_num[i], 0);
-		complex_init (&q, lanczos_denom[i], 0);
+		gnm_complex_init (&p, lanczos_num[i], 0);
+		gnm_complex_init (&q, lanczos_denom[i], 0);
 		while (--i >= 0) {
-			complex_mul (&p, &p, src);
+			gnm_complex_mul (&p, &p, src);
 			p.re += lanczos_num[i];
-			complex_mul (&q, &q, src);
+			gnm_complex_mul (&q, &q, src);
 			q.re += lanczos_denom[i];
 		}
-		complex_div (&pq, &p, &q);
+		gnm_complex_div (&pq, &p, &q);
 
-		complex_init (&zmh, src->re - 0.5, src->im);
-		complex_init (&zmhpg, zmh.re + lanczos_g, zmh.im);
-		complex_init (&zmhd2, zmh.re * 0.5, zmh.im * 0.5);
-		complex_pow (&f, &zmhpg, &zmhd2);
+		gnm_complex_init (&zmh, src->re - 0.5, src->im);
+		gnm_complex_init (&zmhpg, zmh.re + lanczos_g, zmh.im);
+		gnm_complex_init (&zmhd2, zmh.re * 0.5, zmh.im * 0.5);
+		gnm_complex_pow (&f, &zmhpg, &zmhd2);
 
 		zmh.re = -zmh.re; zmh.im = -zmh.im;
-		complex_exp (&f2, &zmh);
-		complex_mul (&f2, &f, &f2);
-		complex_mul (&f2, &f2, &f);
+		gnm_complex_exp (&f2, &zmh);
+		gnm_complex_mul (&f2, &f, &f2);
+		gnm_complex_mul (&f2, &f2, &f);
 
-		complex_mul (dst, &f2, &pq);
+		gnm_complex_mul (dst, &f2, &pq);
 	}
 }
 
 /* ------------------------------------------------------------------------- */
 
 void
-complex_fact (complex_t *dst, complex_t const *src)
+complex_fact (gnm_complex *dst, gnm_complex const *src)
 {
-	if (complex_real_p (src)) {
-		complex_init (dst, gnm_fact (src->re), 0);
+	if (gnm_complex_real_p (src)) {
+		gnm_complex_init (dst, gnm_fact (src->re), 0);
 	} else {
 		/*
 		 * This formula is valid for all arguments except zero
 		 * which we conveniently handled above.
 		 */
-		complex_t gz;
+		gnm_complex gz;
 		complex_gamma (&gz, src);
-		complex_mul (dst, &gz, src);
+		gnm_complex_mul (dst, &gz, src);
 	}
 }
 
 /* ------------------------------------------------------------------------- */
 
+// D(a,z) := z^a * exp(-z) / Gamma (a + 1)
 static void
-igamma_cf (complex_t *dst, const complex_t *a, const complex_t *z)
+complex_temme_D (gnm_complex *res, gnm_complex const *a, gnm_complex const *z)
 {
-	complex_t A0, A1, B0, B1;
-	int i;
+	gnm_complex t, ez, fa;
+
+	// The idea here is to control intermediate sizes and to avoid
+	// accuracy problems caused by exp and pow.  For now, do neither.
+
+	gnm_complex_pow (&t, z, a);
+	gnm_complex_exp (&ez, z);
+	gnm_complex_div (&t, &t, &ez);
+	complex_fact (&fa, a);
+	gnm_complex_div (res, &t, &fa);
+}
+
+
+typedef void (*GnmComplexContinuedFraction) (gnm_complex *ai, gnm_complex *bi,
+					     size_t i, const gnm_complex *args);
+
+static gboolean
+gnm_complex_continued_fraction (gnm_complex *dst, size_t N,
+				GnmComplexContinuedFraction cf,
+				gnm_complex const *args)
+{
+	gnm_complex A0, A1, B0, B1;
+	size_t i;
 	const gboolean debug_cf = FALSE;
 
-	complex_init (&A0, 1, 0);
-	complex_init (&A1, 0, 0);
-	complex_init (&B0, 0, 0);
-	complex_init (&B1, 1, 0);
+	gnm_complex_init (&A0, 1, 0);
+	gnm_complex_init (&A1, 0, 0);
+	gnm_complex_init (&B0, 0, 0);
+	gnm_complex_init (&B1, 1, 0);
 
-	for (i = 1; i < 100; i++) {
-		complex_t ai, bi, t1, t2, c1, c2, A2, B2;
+	for (i = 1; i < N; i++) {
+		gnm_complex ai, bi, t1, t2, c1, c2, A2, B2;
 		gnm_float m;
 		const gnm_float BIG = GNM_const(18446744073709551616.0);
 
-		if (i == 1)
-			complex_init (&ai, 1, 0);
-		else if (i & 1) {
-			gnm_float f = (i >> 1);
-			complex_init (&ai, z->re * f, z->im * f);
-		} else {
-			complex_t f;
-			complex_init (&f, -(a->re + ((i >> 1) - 1)), -a->im);
-			complex_mul (&ai, &f, z);
-		}
-		complex_init (&bi, a->re + (i - 1), a->im);
+		cf (&ai, &bi, i, args);
 
 		/* Update A. */
-		complex_mul (&t1, &bi, &A1);
-		complex_mul (&t2, &ai, &A0);
-		complex_add (&A2, &t1, &t2);
+		gnm_complex_mul (&t1, &bi, &A1);
+		gnm_complex_mul (&t2, &ai, &A0);
+		gnm_complex_add (&A2, &t1, &t2);
 		A0 = A1; A1 = A2;
 
 		/* Update B. */
-		complex_mul (&t1, &bi, &B1);
-		complex_mul (&t2, &ai, &B0);
-		complex_add (&B2, &t1, &t2);
+		gnm_complex_mul (&t1, &bi, &B1);
+		gnm_complex_mul (&t2, &ai, &B0);
+		gnm_complex_add (&B2, &t1, &t2);
 		B0 = B1; B1 = B2;
 
 		/* Rescale */
@@ -1304,99 +1379,242 @@ igamma_cf (complex_t *dst, const complex_t *a, const complex_t *z)
 		if (m >= BIG || m <= 1 / BIG) {
 			int e;
 			gnm_float s;
+
+			if (m == 0)
+				return FALSE;
+
 			(void)frexp (m, &e);
+			if (debug_cf)
+				g_printerr ("rescale by 2^%d\n", -e);
+
 			s = ldexp (1, -e);
 			A0.re *= s; A0.im *= s;
 			A1.re *= s; A1.im *= s;
 			B0.re *= s; B0.im *= s;
 			B1.re *= s; B1.im *= s;
-			if (debug_cf)
-				g_printerr ("rescale\n");
 		}
 
 		/* Check for convergence */
-		complex_mul (&t1, &A1, &B0);
-		complex_mul (&t2, &A0, &B1);
-		complex_sub (&c1, &t1, &t2);
+		gnm_complex_mul (&t1, &A1, &B0);
+		gnm_complex_mul (&t2, &A0, &B1);
+		gnm_complex_sub (&c1, &t1, &t2);
 
-		complex_mul (&c2, &B0, &B1);
+		gnm_complex_mul (&c2, &B0, &B1);
 
-		complex_div (&t1, &A1, &B1);
+		gnm_complex_div (&t1, &A1, &B1);
 		if (debug_cf) {
 			g_printerr ("  a : %.20g + %.20g I\n", ai.re, ai.im);
 			g_printerr ("  b : %.20g + %.20g I\n", bi.re, bi.im);
 			g_printerr ("  A : %.20g + %.20g I\n", A1.re, A1.im);
 			g_printerr ("  B : %.20g + %.20g I\n", B1.re, B1.im);
-			g_printerr ("%3d : %.20g + %.20g I\n", i, t1.re, t1.im);
+			g_printerr ("%3zd : %.20g + %.20g I\n", i, t1.re, t1.im);
 		}
 
-		if (complex_mod (&c1) <= complex_mod (&c2) * (16 * GNM_EPSILON))
+		if (gnm_complex_mod (&c1) <= gnm_complex_mod (&c2) * (GNM_EPSILON /2))
 			break;
 	}
 
-	if (i == 100) {
+	if (i == N) {
+		g_printerr ("continued fraction failed to converge.\n");
 		/* Make the failure obvious. */
 		dst->re = dst->im = gnm_nan;
-		g_printerr ("igamma_cf not converged\n");
-		return;
+		return FALSE;
 	}
 
-	complex_div (dst, &A1, &B1);
+	gnm_complex_div (dst, &A1, &B1);
+	return TRUE;
 }
 
-
-void
-complex_igamma (complex_t *dst, const complex_t *a, const complex_t *z,
-		gboolean lower, gboolean regularized)
+static void
+igamma_lower_coefs (gnm_complex *ai, gnm_complex *bi, size_t i,
+		    gnm_complex const *args)
 {
-	complex_t res, f, mz;
+	gnm_complex const *a = args + 0;
+	gnm_complex const *z = args + 1;
 
-	if (complex_zero_p (a)) {
-		if (!lower && !regularized)
-			complex_gamma (dst, z);
-		else
-			complex_init (dst, lower ? 0 : 1, 0);
-		return;
+	if (i == 1)
+		gnm_complex_real (ai, 1);
+	else if (i & 1) {
+		gnm_complex f;
+		gnm_complex_real (&f, i >> 1);
+		gnm_complex_mul (ai, &f, z);
+	} else {
+		gnm_complex f;
+		gnm_complex_init (&f, -(a->re + ((i >> 1) - 1)), -a->im);
+		gnm_complex_mul (ai, &f, z);
 	}
 
-	if (complex_real_p (a) && a->re >= 0 &&
-	    complex_real_p (z) && z->re >= 0) {
-		complex_init (&res, pgamma (z->re, a->re, 1, lower, FALSE), 0);
-		if (!regularized) {
-			complex_t g;
-			complex_gamma (&g, a);
-			complex_mul (&res, &res, &g);
-		}
-		*dst = res;
-		return;
-	}
+	gnm_complex_init (bi, a->re + (i - 1), a->im);
+}
 
-	igamma_cf (&res, a, z);
+static gboolean
+igamma_lower_cf (gnm_complex *dst, const gnm_complex *a, const gnm_complex *z)
+{
+	gnm_complex args[2] = { *a, *z };
+	gnm_complex f, mz, res;
 
-	/*
-	 * FIXME: The following three blocks should be handled without
-	 * creating big numbers.
-	 */
+	if (!gnm_complex_continued_fraction (&res, 100, igamma_lower_coefs, args))
+		return FALSE;
+
+	// FIXME: The following should be handled without creating big numbers.
 
        	mz.re = -z->re, mz.im = -z->im;
-	complex_exp (&f, &mz);
-	complex_mul (&res, &res, &f);
-	complex_pow (&f, z, a);
-	complex_mul (&res, &res, &f);
+	gnm_complex_exp (&f, &mz);
+	gnm_complex_mul (&res, &res, &f);
+	gnm_complex_pow (&f, z, a);
+	gnm_complex_mul (dst, &res, &f);
 
-	if (!regularized && lower) {
-		/* Nothing */
-	} else {
-		complex_t g;
-		complex_gamma (&g, a);
+	return TRUE;
+}
 
-		if (regularized) {
-			complex_div (&res, &res, &g);
-			if (!lower)
-				res.re = 1 - res.re;
+static gboolean
+igamma_upper_asymp (gnm_complex *dst, const gnm_complex *a, const gnm_complex *z)
+{
+	gnm_float am = gnm_complex_mod (a);
+	gnm_float zm = gnm_complex_mod (z);
+	gnm_float n0;
+	gnm_complex s, t, am1;
+	gboolean debug = FALSE;
+	size_t i;
+
+	if (am >= zm)
+		return FALSE;
+
+	// Things start to diverge here
+	n0 = a->re + gnm_sqrt (zm * zm - a->im * a->im);
+	if (debug)
+		g_printerr ("n0=%g\n", n0);
+	(void)n0;
+
+	// FIXME: Verify this condition for whether we have enough
+	// precision at term n0
+	if (2 * zm < GNM_MANT_DIG * M_LN2gnum)
+		return FALSE;
+
+	gnm_complex_real (&s, 0);
+
+	gnm_complex_init (&am1, a->re - 1, a->im);
+	complex_temme_D (&t, &am1, z);
+
+	for (i = 0; i < 100; i++) {
+		gnm_complex api;
+
+		gnm_complex_add (&s, &s, &t);
+
+		if (debug) {
+			g_printerr ("%3zd: t=%.20g + %.20g * I\n", i, t.re, t.im);
+			g_printerr ("   : s=%.20g + %.20g * I\n", s.re, s.im);
+		}
+
+		if (gnm_complex_mod (&t) <= gnm_complex_mod (&s) * GNM_EPSILON) {
+			if (debug)
+				g_printerr ("igamma_upper_asymp converged.\n");
+			*dst = s;
+			return TRUE;
+		}
+
+		gnm_complex_div (&t, &t, z);
+		gnm_complex_init (&api, a->re - (i + 1), a->im);
+		gnm_complex_mul (&t, &t, &api);
+	}
+
+	if (debug)
+		g_printerr ("igamma_upper_asymp failed to converge.\n");
+
+	return FALSE;
+}
+
+static void
+fixup_upper_real (gnm_complex *res, gnm_complex const *a, gnm_complex const *z)
+{
+	// This assumes we have an upper gamma regularized result.
+	//
+	// It appears that upper algorithms have trouble with negative real z
+	// (for example, such z being outside the allowed domain) in some cases.
+	
+
+	if (gnm_complex_real_p (z) && z->re < 0 &&
+	    gnm_complex_real_p (a) && a->re != gnm_floor (a->re)) {
+		// Everything in the lower power series is real expect z^a
+		// which is not.  So...
+		// 1. Flip to lower gamma
+		// 2. Assume modulus is correct
+		// 3. Use exact angle for lower gamma
+		// 4. Flip back to upper gamma
+		gnm_complex lres = *res;
+		lres.re = 1 - lres.re;
+
+		gnm_complex_from_polar_pi (res,
+					   gnm_complex_mod (&lres),
+					   a->re);
+		res->re = 1 - res->re;
+		res->im = 0 - res->im;
+	}
+}
+
+void
+complex_igamma (gnm_complex *dst, const gnm_complex *a, const gnm_complex *z,
+		gboolean lower, gboolean regularized)
+{
+	gnm_complex res, ga;
+	gboolean have_lower, have_regularized;
+	gboolean have_ga = FALSE;
+
+	if (regularized && gnm_complex_real_p (a) &&
+	    a->re <= 0 && a->re == gnm_floor (a->re)) {
+		gnm_complex_real (&res, 0);
+		have_lower = FALSE;
+		have_regularized = TRUE;
+		goto fixup;
+	}
+
+	if (gnm_complex_real_p (a) && a->re >= 0 &&
+	    gnm_complex_real_p (z) && z->re >= 0) {
+		gnm_complex_init (&res, pgamma (z->re, a->re, 1, lower, FALSE), 0);
+		have_lower = lower;
+		have_regularized = TRUE;
+		goto fixup;
+	}
+
+	if (igamma_upper_asymp (&res, a, z)) {
+		have_lower = FALSE;
+		have_regularized = TRUE;
+		fixup_upper_real (&res, a, z);
+		goto fixup;
+	}
+
+	if (igamma_lower_cf (&res, a, z)) {
+		have_lower = TRUE;
+		have_regularized = FALSE;
+		goto fixup;
+	}
+
+	gnm_complex_init (dst, gnm_nan, gnm_nan);
+	return;
+
+fixup:
+	// Fixup to the desired form as needed.  This is not ideal.
+	// 1. Regularizing here is too late due to overflow.
+	// 2. Upper/lower switch can have cancellation
+	if (regularized != have_regularized) {
+		complex_gamma (&ga, a);
+		have_ga = TRUE;
+
+		if (have_regularized)
+			gnm_complex_mul (&res, &res, &ga);
+		else
+			gnm_complex_div (&res, &res, &ga);
+		have_regularized = TRUE;
+	}
+
+	if (lower != have_lower) {
+		if (have_regularized) {
+			res.re = 1 - res.re;
+			res.im = 0 - res.im;
 		} else {
-			/* !lower here */
-			complex_sub (&res, &g, &res);
+			if (!have_ga)
+				complex_gamma (&ga, a);
+			gnm_complex_sub (&res, &ga, &res);
 		}
 	}
 
