@@ -207,6 +207,7 @@ my @full_corpus =
      "$samples/cell-comment-tests.gnumeric",
      "$samples/colrow-tests.gnumeric",
      "$samples/cond-format-tests.gnumeric",
+     "$samples/format-tests.gnumeric",
      "$samples/formula-tests.gnumeric",
      "$samples/graph-tests.gnumeric",
      "$samples/merge-tests.gnumeric",
@@ -216,6 +217,7 @@ my @full_corpus =
      "$samples/page-setup-tests.gnumeric",
      "$samples/rich-text-tests.gnumeric",
      "$samples/sheet-formatting-tests.gnumeric",
+     "$samples/sheet-names-tests.gnumeric",
      "$samples/sheet-tab-tests.gnumeric",
      "$samples/solver-tests.gnumeric",
      "$samples/split-panes-tests.gnumeric",
